Package: NeutroBasicDesignsAnalysis
Type: Package
Title: Neutrosophic Analysis of Completely Randomized Designs and
        Randomized Complete Block Designs
Version: 0.0.1
Authors@R: c(
    person(
      "Neethu", "R.S.",
      email = "neethumpras37@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Boyina", "Devi Priyanka",
      email = "boyinadevipriyanka@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Cini", "Varghese",
      email = "cini2204@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Susheel Kumar", "Sarkar",
      email = "suskar982@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Mohd", "Harun",
      email = "harun.agribhu@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
  "Vinaykumar", "L.N.",
  email = "vinaymandya123@gmail.com",
  role = c("aut", "cre")
   ),
    person(
      "Anindita", "Datta",
      email = "aninditaiasri@gmail.com",
      role = c("aut", "ctb")
    )
  )
Maintainer: Vinaykumar L.N. <vinaymandya123@gmail.com>
Description: Provides neutrosophic statistical methods for interval-valued
    data from completely randomized and randomized complete block designs.
    Methods include neutrosophic analysis of variance, analysis of covariance,
    multivariate analysis of variance, pooled analysis of variance, Levene's
    test, and Aitken transformation. When the lower and upper bounds are equal
    (crisp data), the methods reduce to their corresponding classical
    statistical analyses. The basic concept of neutrosophic statistics is based
    on Smarandache (2014) <https://fs.unm.edu/NeutrosophicStatistics.pdf>,
    while the statistical analysis procedures implemented in this package are
    newly developed.
License: GPL (>= 3)
Encoding: UTF-8
Imports: MASS, stats
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-05 04:16:59 UTC; admin
Author: Neethu R.S. [aut, ctb],
  Boyina Devi Priyanka [aut, ctb],
  Cini Varghese [aut, ctb],
  Susheel Kumar Sarkar [aut, ctb],
  Mohd Harun [aut, ctb],
  Vinaykumar L.N. [aut, cre],
  Anindita Datta [aut, ctb]
Repository: CRAN
Date/Publication: 2026-08-09 08:40:13 UTC
Built: R 4.6.1; ; 2026-08-09 12:55:41 UTC; unix
