# Generated by roxygen2: do not edit by hand

export(CRDnsANCOVA)
export(CRDnsANOVA)
export(CRDnsMANOVA)
export(PooledCRDnsANOVA)
export(PooledRBDnsANOVA)
export(RCBDnsANCOVA)
export(RCBDnsANOVA)
export(nsAitkenTransform)
export(nsLeveneTest)
importFrom(MASS,ginv)
importFrom(stats,model.matrix)
importFrom(stats,qf)
importFrom(stats,qt)
