% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CRDnsANOVA.R
\name{CRDnsANOVA}
\alias{CRDnsANOVA}
\title{Neutrosophic Analysis of Variance for Completely Randomized Design}
\usage{
CRDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Matrix of lower bounds of the response variable.}

\item{Upper_y}{Matrix of upper bounds of the response variable.}

\item{design}{Matrix specifying treatment allocation.}

\item{alpha}{Significance level for LSD test.}

\item{verbose}{Logical. If TRUE, prints the analysis.}
}
\value{
A list containing the NANOVA table, treatment means,
pairwise comparisons, and LSD interval.
}
\description{
Performs neutrosophic analysis of variance (NANOVA) for a
completely randomized design using interval-valued observations.
}
\examples{
Lower_y <- matrix(c(
  9.5, 19.5, 29.5, 39.5,
  10.0, 20.0, 30.0, 40.0,
  10.5, 20.5, 30.5, 40.5,
  9.8, 19.8, 29.8, 39.8,
  10.2, 20.2, 30.2, 40.2
), nrow = 5, byrow = TRUE)

Upper_y <- matrix(c(
  10.5, 20.5, 30.5, 40.5,
  11.0, 21.0, 31.0, 41.0,
  11.5, 21.5, 31.5, 41.5,
  10.8, 20.8, 30.8, 40.8,
  11.2, 21.2, 31.2, 41.2
), nrow = 5, byrow = TRUE)

design <- matrix(c(
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4
), nrow = 5, byrow = TRUE)

CRDnsANOVA(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  design = design,
  alpha = 0.05,
  verbose = TRUE
)
}
