% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RBDnsANOVA.R
\name{RCBDnsANOVA}
\alias{RCBDnsANOVA}
\title{Neutrosophic Analysis of Variance for Randomized Complete Block Design}
\usage{
RCBDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower bounds of the response variable.}

\item{Upper_y}{Numeric matrix of upper bounds of the response variable.}

\item{design}{Numeric matrix representing the RCBD layout.}

\item{alpha}{Significance level for interval-based LSD test. Default is 0.05.}

\item{verbose}{Logical. If \code{TRUE}, prints the NANOVA table and LSD comparisons.}
}
\value{
A list containing the Neutrosophic ANOVA table, interval-based LSD
comparisons (if applicable), and the interval LSD.
}
\description{
Performs Neutrosophic Analysis of Variance (NANOVA) for interval-valued
response data from a Randomized Complete Block Design (RCBD).
}
\examples{
Lower_y <- matrix(c(
  120.230,125.488,132.987,127.086,127.672,128.013,
  122.594,121.009,123.969,120.358,120.424,122.197,
  121.183,130.671,128.794,114.863,122.595,122.073,
  127.620,124.532,132.893,125.528,125.850,127.550
), nrow = 4, byrow = TRUE)

Upper_y <- matrix(c(
  127.6967536,131.2116955,141.2127373,136.1540904,130.6884772,136.8474149,
  129.8264289,130.3314544,133.3113414,126.5063118,128.4362999,130.2714433,
  124.5068016,139.3287297,134.1060197,124.2774447,127.2248520,130.3948469,
  131.0638721,129.8884785,135.5666716,127.7580663,132.0178679,133.3903886
), nrow = 4, byrow = TRUE)

design <- matrix(c(
  1,2,3,4,5,6,
  1,2,3,4,5,6,
  1,2,3,4,5,6,
  1,2,3,4,5,6
), nrow = 4, byrow = TRUE)

RCBDnsANOVA(Lower_y, Upper_y, design)

}
