% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RBDnsANCOVA.R
\name{RCBDnsANCOVA}
\alias{RCBDnsANCOVA}
\title{Neutrosophic Analysis of Covariance for RCBD}
\usage{
RCBDnsANCOVA(
  Lower_y,
  Upper_y,
  Lower_z,
  Upper_z,
  design,
  alpha = 0.05,
  verbose = FALSE
)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower response values.}

\item{Upper_y}{Numeric matrix of upper response values.}

\item{Lower_z}{Numeric matrix of lower covariate values.}

\item{Upper_z}{Numeric matrix of upper covariate values.}

\item{design}{Numeric matrix representing the RCBD layout.}

\item{alpha}{Significance level for LSD test. Default is \code{0.05}.}

\item{verbose}{Logical; if \code{TRUE}, prints the analysis. Default is \code{FALSE}.}
}
\value{
A list containing the NANCOVA table, LSD interval, and treatment
comparisons (if treatment effect is significant).
}
\description{
Performs neutrosophic analysis of covariance for a Randomized Complete
Block Design using interval-valued response and covariate data.
}
\examples{
Lower_y <- matrix(c(
  46.84, 97.69, 39.30, 49.20,
  51.52, 83.38, 56.48, 42.15,
  44.42, 49.64, 44.21, 35.79,
  32.77, 74.30, 55.72, 70.55
), nrow = 4, byrow = TRUE)

Upper_y <- matrix(c(
  49.66,101.31,47.70,55.80,
  60.98,87.62,60.52,44.85,
  52.26,59.36,52.79,44.61,
  42.23,82.70,59.28,75.45
), nrow = 4, byrow = TRUE)

Lower_z <- matrix(c(
  224.95,245.87,245.19,259.41,
  222.06,213.83,253.10,247.51,
  255.67,230.55,265.05,246.67,
  137.98,214.68,251.49,257.70
), nrow = 4, byrow = TRUE)

Upper_z <- matrix(c(
  229.05,250.13,252.81,268.59,
  229.94,222.17,258.90,256.49,
  262.33,237.45,274.95,249.33,
  142.02,219.32,260.51,264.30
), nrow = 4, byrow = TRUE)

design <- matrix(c(
  1,2,3,4,
  1,2,3,4,
  1,2,3,4,
  1,2,3,4
), nrow = 4, byrow = TRUE)

result <- RCBDnsANCOVA(
  Lower_y,
  Upper_y,
  Lower_z,
  Upper_z,
  design,
  alpha = 0.05,
  verbose = TRUE
)
}
